"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
  This base128 algorithm is based on Mikael Grev's MiGBase64 algorithm: http://migbase64.sourceforge.net
  which is licensed under the BSD Open Source license.
 */
var Long = require("long");
var constants_1 = require("./constants");
var shared_1 = require("../shared");
exports.REGEXP = /^[/-9?-Za-z¿-ý]{2,}$/;
var CODES = new Array(0x100);
Array.from(constants_1.SYMBOLS, shared_1.toCharCode)
    .forEach(function (code, i) { return CODES[code] = shared_1.LONG_BYTES[i]; });
/**
 * Expects a non-empty string value.
 */
function decode(encoded) {
    var length = encoded.length;
    var bytesCount = Math.trunc(length * constants_1.WORD_SIZE / constants_1.BYTE_SIZE);
    var fullWordsEnd = Math.trunc(bytesCount / constants_1.WORD_SIZE) * constants_1.WORD_SIZE;
    var result = new Uint8Array(bytesCount);
    var charPos = 0;
    var bytePos = 0;
    while (bytePos < fullWordsEnd) {
        var unpacked = Long.ZERO;
        for (var shift = constants_1.WORD_SHIFT_START; shift > -1; shift -= constants_1.WORD_SIZE) {
            unpacked = unpackChar(encoded, charPos++, unpacked, shift);
        }
        for (var shift = constants_1.BYTE_SHIFT_START; shift > -1; shift -= constants_1.BYTE_SIZE) {
            result[bytePos++] = unpackByte(unpacked, shift);
        }
    }
    // remainder
    if (bytePos < bytesCount) {
        var unpacked = Long.ZERO;
        for (var shift = constants_1.WORD_SHIFT_START; charPos < length; shift -= constants_1.WORD_SIZE) {
            unpacked = unpackChar(encoded, charPos++, unpacked, shift);
        }
        for (var shift = constants_1.BYTE_SHIFT_START; bytePos < bytesCount; shift -= constants_1.BYTE_SIZE) {
            result[bytePos++] = unpackByte(unpacked, shift);
        }
    }
    return result;
}
exports.decode = decode;
function unpackChar(encoded, charPos, unpacked, shift) {
    var charCode = encoded.charCodeAt(charPos);
    var value = CODES[charCode];
    if (!value) {
        throw new Error("invalid character code: '" + charCode + "' at position " + charPos);
    }
    return unpacked.or(value.shiftLeft(shift));
}
function unpackByte(unpacked, shift) {
    return unpacked.shiftRight(shift).low;
}
//# sourceMappingURL=decode.js.map