"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
  This base32 algorithm is based on Mikael Grev's MiGBase64 algorithm: http://migbase64.sourceforge.net
  which is licensed under the BSD Open Source license.
 */
var Long = require("long");
var constants_1 = require("./constants");
var shared_1 = require("../shared");
var BITS_MASK = 0x1f;
var CODES = Array.from(constants_1.SYMBOLS, shared_1.toCharCode);
var CHECK_CODES = CODES.concat(Array.from(constants_1.CHECK_EXTRAS, shared_1.toCharCode));
function encode(unencoded) {
    var wordCount = unencoded.length / constants_1.WORD_SIZE;
    var charCount = Math.ceil(wordCount * constants_1.BYTE_SIZE) + 1; // + 1 is check digit
    var fullWordsEnd = Math.trunc(wordCount) * constants_1.WORD_SIZE;
    var result = new Array(charCount);
    var charPos = 0;
    var bytePos = 0;
    var checksum = 0;
    while (bytePos < fullWordsEnd) {
        var packed = Long.ZERO;
        for (var shift = constants_1.BYTE_SHIFT_START; shift > -1; shift -= constants_1.BYTE_SIZE) {
            var byte = unencoded[bytePos++];
            packed = packByte(byte, packed, shift);
            checksum += byte;
        }
        for (var shift = constants_1.WORD_SHIFT_START; shift > -1; shift -= constants_1.WORD_SIZE) {
            result[charPos++] = packChar(packed, shift);
        }
    }
    // remainder
    if (bytePos < unencoded.length) {
        var packed = Long.ZERO;
        for (var shift_1 = constants_1.BYTE_SHIFT_START; bytePos < unencoded.length; shift_1 -= constants_1.BYTE_SIZE) {
            var byte = unencoded[bytePos++];
            packed = packByte(byte, packed, shift_1);
            checksum += byte;
        }
        // this is different from Base128 because it is possible to have more than one symbol in a byte.
        var remainder = unencoded.length - fullWordsEnd;
        var shift = constants_1.WORD_SHIFT_START;
        result[charPos++] = packChar(packed, shift);
        result[charPos++] = packChar(packed, shift -= constants_1.WORD_SIZE);
        if (remainder > 1) {
            result[charPos++] = packChar(packed, shift -= constants_1.WORD_SIZE);
            result[charPos++] = packChar(packed, shift -= constants_1.WORD_SIZE);
        }
        if (remainder > 2) {
            result[charPos++] = packChar(packed, shift -= constants_1.WORD_SIZE);
        }
        if (remainder > 3) {
            result[charPos++] = packChar(packed, shift - constants_1.WORD_SIZE);
            result[charPos++] = packChar(packed, constants_1.WORD_SIZE);
        }
    }
    result[charPos] = CHECK_CODES[checksum % constants_1.CHECK_PRIME];
    return String.fromCharCode.apply(String, result);
}
exports.encode = encode;
function packByte(byte, packed, shift) {
    return packed.or(shared_1.LONG_BYTES[byte].shiftLeft(shift));
}
function packChar(packed, shift) {
    return CODES[packed.shiftRight(shift).low & BITS_MASK];
}
//# sourceMappingURL=encode.js.map