"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var S = require("js.spec");
var msgpack = require("msgpack-typed-numbers");
var decode128 = require("./base128/decode");
var decode32 = require("./base32/decode");
var finder_1 = require("./finder");
var shared_1 = require("./shared");
var factory_1 = require("./factory");
/**
 * Convert an encoded identifier string into an Identifier. The string can be in either data or human format.
 * @param encoded the encoded string
 * @returns an Identifier instance
 */
function decodeFromString(encoded) {
    var bytes = decodeString(encoded);
    var idTuple = decodeBytes(bytes);
    return decodeToIdentifier(idTuple);
}
exports.decodeFromString = decodeFromString;
function decodeToIdentifier(tuple) {
    var typeCode = tuple[0], decoded = tuple[1];
    var codec = finder_1.codecForTypeCode(typeCode);
    var value = decodeWithCodec(codec, decoded);
    return factory_1.createIdentifier(codec, value);
}
exports.decodeToIdentifier = decodeToIdentifier;
var stringSpec = S.spec.and("encoded identifier", S.spec.predicate("encoded string", S.spec.string), S.spec.predicate("not empty", function (value) { return shared_1.exists(value) && value.length > 0; }));
function decodeString(encoded) {
    S.assert(stringSpec, encoded);
    // check for encoded msgpack 2-element array
    var firstChar = encoded.charCodeAt(0);
    if (firstChar === 0xc7) { // Ç
        return decode128.decode(encoded);
    }
    if (firstChar === 0x4a || firstChar === 0x6a) { // j/J
        return decode32.decode(encoded);
    }
    throw new Error("Not a valid encoded identifier: '" + encoded + "'");
}
exports.decodeString = decodeString;
function decodeBytes(bytes) {
    var decoded = msgpack.decode(bytes);
    S.assert(shared_1.decodedIdSpec, decoded);
    return decoded;
}
exports.decodeBytes = decodeBytes;
function decodeWithCodec(codec, encoded) {
    S.assert(codec.specForDecoding, encoded);
    return codec.decode(encoded);
}
exports.decodeWithCodec = decodeWithCodec;
//# sourceMappingURL=decode.js.map