"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var msgpack = require("msgpack-typed-numbers");
var base128 = require("./base128/encode");
var base32 = require("./base32/encode");
var finder_1 = require("./finder");
/**
 * Convert an Identifier into a base-128 encoded identifier string.
 * @param identifier the identifier object
 * @returns an encoded identifier string
 */
function toBase128String(identifier) {
    var bytes = encodeToBytes(identifier);
    return base128.encode(bytes);
}
exports.toBase128String = toBase128String;
/**
 * Convert an Identifier into a base-32 encoded identifier string.
 * @param identifier the identifier object
 * @returns an encoded identifier string
 */
function toBase32String(identifier) {
    var bytes = encodeToBytes(identifier);
    return base32.encode(bytes);
}
exports.toBase32String = toBase32String;
/**
 * Convert an Identifier into a debug-friendly string. Presents the type and value of the identifier.
 * @param identifier the identifier object
 * @returns a string representation of the identifier
 */
function toDebugString(identifier) {
    var codec = finder_1.findCodec(identifier);
    return "ID\u00AB" + identifier.type + "\u00BB" + codec.toDebugString(identifier.value);
}
exports.toDebugString = toDebugString;
function encodeToBytes(identifier) {
    var tuple = encodeIdTuple(identifier);
    return encodeBytes(tuple);
}
function encodeIdTuple(identifier) {
    var codec = finder_1.findCodec(identifier);
    var value = encodeWithCodec(codec, identifier.value);
    return [codec.typeCode, value];
}
exports.encodeIdTuple = encodeIdTuple;
function encodeWithCodec(codec, value) {
    return codec.encode(value);
}
exports.encodeWithCodec = encodeWithCodec;
function encodeBytes(tuple) {
    return msgpack.encode(tuple);
}
exports.encodeBytes = encodeBytes;
//# sourceMappingURL=encode.js.map