"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var factory_1 = require("./factory");
var lists_1 = require("./types/lists");
var maps_1 = require("./types/maps");
var finder_1 = require("./finder");
var composite = require("./types/composite");
var string_1 = require("./types/string");
var boolean_1 = require("./types/boolean");
var integer_1 = require("./types/integer");
var float_1 = require("./types/float");
var long_1 = require("./types/long");
var bytes_1 = require("./types/bytes");
var uuid_1 = require("./types/uuid");
var datetime_1 = require("./types/datetime");
var geo_1 = require("./types/geo");
var semantic_1 = require("./semantic");
function processCodec(itemCodec) {
    var listCodec = lists_1.createListCodec(itemCodec);
    var mapCodec = maps_1.createMapCodec(itemCodec);
    finder_1.registerCodec(itemCodec);
    finder_1.registerCodec(listCodec);
    finder_1.registerCodec(mapCodec);
    if ((itemCodec.typeCode & semantic_1.SEMANTIC_TYPE_FLAG) === 0) {
        // primitive type
        var listOfListsCodec = lists_1.createListCodec(listCodec);
        var listOfMapsCodec = lists_1.createListCodec(mapCodec);
        var mapOfListsCodec = maps_1.createMapCodec(listCodec);
        var mapOfMapsCodec = maps_1.createMapCodec(mapCodec);
        finder_1.registerCodec(listOfListsCodec);
        finder_1.registerCodec(listOfMapsCodec);
        finder_1.registerCodec(mapOfListsCodec);
        finder_1.registerCodec(mapOfMapsCodec);
    }
    return factory_1.createFactory(itemCodec, listCodec, mapCodec);
}
exports.processCodec = processCodec;
function createCompositeFactory() {
    finder_1.registerCodec(composite.listCodec);
    finder_1.registerCodec(composite.mapCodec);
    return {
        list: factory_1.createListFactory(composite.listCodec),
        map: factory_1.createMapFactory(composite.mapCodec)
    };
}
exports.createCompositeFactory = createCompositeFactory;
/**
 * Factories for identifiers.
 */
exports.factory = {
    string: processCodec(string_1.stringCodec),
    boolean: processCodec(boolean_1.booleanCodec),
    integer: processCodec(integer_1.integerCodec),
    float: processCodec(float_1.floatCodec),
    long: processCodec(long_1.longCodec),
    bytes: processCodec(bytes_1.bytesCodec),
    uuid: processCodec(uuid_1.uuidCodec),
    datetime: processCodec(datetime_1.datetimeCodec),
    geo: processCodec(geo_1.geoCodec),
    composite: createCompositeFactory()
};
//# sourceMappingURL=factory-constructor.js.map