import { Identifier } from "./identifier";
import { MapIdentifier, TypedObject } from "./shared";
import { CompositeIdList, CompositeIdMap } from "./types/composite";
import { IdentifierCodec } from "./identifier-codec";
export declare type ItemFactory<INPUT, VALUE> = (input: INPUT) => Identifier<VALUE>;
export declare type ListInput<INPUT> = INPUT[] | INPUT;
export declare type ListFactory<INPUT, VALUE> = (...inputs: ListInput<INPUT>[]) => Identifier<VALUE[]>;
export declare type MapFactory<INPUT, VALUE> = (input: TypedObject<INPUT>) => MapIdentifier<VALUE>;
export declare type Factory<INPUT, VALUE, ITEMFACTORY extends ItemFactory<INPUT, VALUE> = ItemFactory<INPUT, VALUE>> = ITEMFACTORY & {
    list: ListFactory<INPUT, VALUE>;
    map: MapFactory<INPUT, VALUE>;
};
export interface CompositeFactory {
    list(...inputs: CompositeIdList): Identifier<CompositeIdList>;
    map(input: CompositeIdMap): Identifier<CompositeIdMap>;
}
export declare function createListFactory<INPUT, VALUE, ENCODED>(listCodec: IdentifierCodec<INPUT[], VALUE[], ENCODED[]>): ListFactory<INPUT, VALUE>;
export declare function createMapFactory<INPUT, VALUE, ENCODED>(mapCodec: IdentifierCodec<TypedObject<INPUT>, TypedObject<VALUE>, TypedObject<ENCODED>>): MapFactory<INPUT, VALUE>;
export declare function createFactory<INPUT, VALUE, ENCODED>(itemCodec: IdentifierCodec<INPUT, VALUE, ENCODED>, listCodec: IdentifierCodec<INPUT[], VALUE[], ENCODED[]>, mapCodec: IdentifierCodec<TypedObject<INPUT>, TypedObject<VALUE>, TypedObject<ENCODED>>): Factory<INPUT, VALUE>;
export declare function createIdentifier<INPUT, VALUE, ENCODED>(codec: IdentifierCodec<INPUT, VALUE, ENCODED>, value: VALUE): Identifier<VALUE>;
