"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var S = require("js.spec");
var encode = require("./encode");
var shared_1 = require("./shared");
function createListFactory(listCodec) {
    return function () {
        var inputs = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            inputs[_i] = arguments[_i];
        }
        //inputs may be an array, but spread operator wraps it in another array. Extract the original array.
        if (inputs.length === 1 && Array.isArray(inputs[0])) {
            inputs = inputs[0];
        }
        return newIdentifier(listCodec, inputs);
    };
}
exports.createListFactory = createListFactory;
function createMapFactory(mapCodec) {
    return function (mapInput) { return newIdentifier(mapCodec, mapInput); };
}
exports.createMapFactory = createMapFactory;
function createFactory(itemCodec, listCodec, mapCodec) {
    var factory = (function (input) { return newIdentifier(itemCodec, input); });
    factory.list = createListFactory(listCodec);
    factory.map = createMapFactory(mapCodec);
    return factory;
}
exports.createFactory = createFactory;
function newIdentifier(codec, input) {
    S.assert(codec.specForIdentifier, input);
    return createIdentifier(codec, codec.forIdentifier(input));
}
function createIdentifier(codec, value) {
    var _a;
    var identifier = (_a = {
            value: value,
            type: codec.type,
            toString: function () { return encode.toDebugString(identifier); },
            toHumanString: function () { return encode.toBase32String(identifier); },
            toDataString: function () { return encode.toBase128String(identifier); },
            toJSON: function () { return encode.toBase128String(identifier); }
        },
        _a[shared_1.codecSymbol] = codec,
        _a);
    return shared_1.deepFreeze(identifier);
}
exports.createIdentifier = createIdentifier;
//# sourceMappingURL=factory.js.map