"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var S = require("js.spec");
var shared_1 = require("./shared");
var semantic_1 = require("./semantic");
var codecs = [];
function registerCodec(codec) {
    codecs[codec.typeCode] = codec;
}
exports.registerCodec = registerCodec;
function codecForTypeCode(typeCode) {
    var codec = codecs[typeCode];
    if (codec) {
        return codec;
    }
    if (typeCode < semantic_1.SEMANTIC_TYPE_FLAG) {
        throw new Error("No codec for typeCode '" + typeCode + "' found.");
    }
    return createUnknownCodec(typeCode);
}
exports.codecForTypeCode = codecForTypeCode;
function createUnknownCodec(typeCode) {
    var baseTypeCode = typeCode & semantic_1.SEMANTIC_TYPE_MASK;
    var baseCodec = codecForTypeCode(baseTypeCode);
    var unknownCodec = __assign({}, baseCodec, { typeCode: typeCode, type: "unknown-" + baseCodec.type });
    registerCodec(unknownCodec);
    return codecForTypeCode(typeCode);
}
function findCodec(identifier) {
    S.assert(shared_1.identifierSpec, identifier);
    // @ts-ignore: codec not part of identifier interface
    var codec = identifier[shared_1.codecSymbol];
    if (Object.is(codec, codecs[codec.typeCode])) {
        return codec;
    }
    throw new Error("unknown codec found on " + JSON.stringify(identifier.value) + " : " + JSON.stringify(codec));
}
exports.findCodec = findCodec;
//# sourceMappingURL=finder.js.map