"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Long = require("long");
var S = require("js.spec");
exports.LONG_BYTES = new Array(0x100);
for (var i = 0; i < 0x100; i++) {
    exports.LONG_BYTES[i] = Long.fromInt(i, false);
}
/**
 * Symbol key to store codec in an identifier instance.
 */
exports.codecSymbol = Symbol.for("id-codec");
/**
 * Common predicate used to test if a value exists--neither undefined or null. Checks for boolean values so that
 * a value of 'false' will return true fo this method because the value exists.
 * @param value the value to test
 * @returns true if the value exists, regardless of it's actual value
 */
function exists(value) {
    return !!value || value != null;
}
exports.exists = exists;
function isString(value) {
    return typeof value === "string";
}
exports.isString = isString;
function isNumber(value) {
    return typeof value === "number";
}
exports.isNumber = isNumber;
/**
 * Recursively deep-freeze objects.
 * @param obj the object to deep freeze
 * @returns the frozen object
 */
function deepFreeze(obj) {
    Object.freeze(obj);
    for (var _i = 0, _a = Object.getOwnPropertyNames(obj); _i < _a.length; _i++) {
        var prop = _a[_i];
        var value = obj[prop];
        if (value !== null
            && typeof value === "object"
            && !Object.isFrozen(value)) {
            deepFreeze(value);
        }
    }
    return obj;
}
exports.deepFreeze = deepFreeze;
/**
 * Convenience function to extract the first character code from a string.
 */
function toCharCode(char) {
    return char.charCodeAt(0);
}
exports.toCharCode = toCharCode;
var codecAssignedSpec = S.spec.predicate("codec assigned", S.spec.object);
function hasCodecSymbol(identifier) {
    // @ts-ignore: codec not part of identifier interface
    return S.valid(codecAssignedSpec, identifier[exports.codecSymbol]);
}
exports.identifierSpec = S.spec.and("identifier", hasCodecSymbol, S.spec.map("identifier structure", {
    type: S.spec.string,
    value: exists
}));
exports.decodedIdSpec = S.spec.tuple("decoded identifier array", Number.isInteger, exists);
//# sourceMappingURL=shared.js.map