"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var S = require("js.spec");
var shared_types_1 = require("./shared-types");
var shared_1 = require("../shared");
function isArrayBuffer(input) {
    return input instanceof ArrayBuffer;
}
function isArrayLike(input) {
    return typeof input === "object"
        && shared_1.isNumber(input.length);
}
function forBytesIdentifier(input) {
    // copies the input content
    return Array.from(isArrayBuffer(input)
        ? new Uint8Array(input) // creates a view, does not copy the buffer
        : input);
}
exports.forBytesIdentifier = forBytesIdentifier;
function isValidType(input) {
    return isArrayBuffer(input) || isArrayLike(input);
}
function isValidLength(input, minLen, maxLen) {
    var len = isArrayBuffer(input)
        ? input.byteLength
        : input.length;
    return len >= minLen && len <= maxLen;
}
exports.isValidLength = isValidLength;
function containsOnlyBytes(input) {
    for (var pos = 0; pos < input.length; pos++) {
        var byte = input[pos];
        if (!Number.isInteger(byte) || (byte < 0x0 || byte > 0xff)) {
            return false;
        }
    }
    return true;
}
exports.bytesInputSpec = S.spec.and("bytes identifier", isValidType, containsOnlyBytes);
function isUint8Array(value) {
    return value instanceof Uint8Array;
}
exports.bytesDecodingSpec = S.spec.predicate("bytes decoding", isUint8Array);
function encodeBytes(value) {
    return new Uint8Array(value);
}
function decodeBytes(decoded) {
    return Array.from(decoded);
}
exports.bytesCodec = {
    type: "bytes",
    typeCode: 0x5,
    specForIdentifier: exports.bytesInputSpec,
    specForDecoding: exports.bytesDecodingSpec,
    forIdentifier: forBytesIdentifier,
    toDebugString: shared_types_1.asIsCodec.toDebugString,
    encode: encodeBytes,
    decode: decodeBytes
};
//# sourceMappingURL=bytes.js.map