"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var shared_1 = require("../shared");
var lists_1 = require("./lists");
var S = require("js.spec");
var shared_types_1 = require("./shared-types");
var decode_1 = require("../decode");
var encode_1 = require("../encode");
var maps_1 = require("./maps");
var COMPOSITE_TYPE_CODE = 0x18;
var specForListInput = S.spec.and("composite-list forIdentifier", S.spec.array, // must be an array, not a Set
S.spec.collection("identifier item spec", shared_1.identifierSpec));
var specForDecodingList = S.spec.and("composite-list decode", S.spec.array, // must be an array, not a Set
S.spec.collection("decoded identifier item", shared_1.decodedIdSpec));
function toDebugStringList(list) {
    var joined = list
        .map(function (value) { return value.toString(); })
        .join(", ");
    return "[" + joined + "]";
}
function encodeList(list) {
    return list.map(encode_1.encodeIdTuple);
}
function decodeList(list) {
    return list.map(decode_1.decodeToIdentifier);
}
exports.listCodec = {
    type: "composite-list",
    typeCode: COMPOSITE_TYPE_CODE | lists_1.LIST_OF,
    specForIdentifier: specForListInput,
    specForDecoding: specForDecodingList,
    forIdentifier: shared_types_1.asIsCodec.forIdentifier,
    toDebugString: toDebugStringList,
    encode: encodeList,
    decode: decodeList
};
var specForMapInput = S.spec.and("composite-map forIdentifier spec", S.spec.object, new maps_1.MapValuesSpec(shared_1.identifierSpec, "Map identifier values"));
var specForDecodingMap = S.spec.and("composite-map decode spec", S.spec.object, new maps_1.MapValuesSpec(shared_1.decodedIdSpec, "Map identifier values"));
function toDebugStringMap(map) {
    var keys = Object.keys(map);
    var joined = keys
        .map(function (key) { return key + ": " + map[key].toString(); })
        .join(", ");
    return "{" + joined + "}";
}
function forMapIdentifier(map) {
    return maps_1.mapValues(map, shared_types_1.asIsCodec.forIdentifier, true);
}
function encodeMap(map) {
    return maps_1.mapValues(map, encode_1.encodeIdTuple);
}
function decodeMap(map) {
    return maps_1.mapValues(map, decode_1.decodeToIdentifier);
}
exports.mapCodec = {
    type: "composite-map",
    typeCode: COMPOSITE_TYPE_CODE | maps_1.MAP_OF,
    specForIdentifier: specForMapInput,
    specForDecoding: specForDecodingMap,
    forIdentifier: forMapIdentifier,
    toDebugString: toDebugStringMap,
    encode: encodeMap,
    decode: decodeMap
};
//# sourceMappingURL=composite.js.map