"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
  Geolocation in decimal degrees for latitude and longitude. Conforms to the World Geodetic System (WGS).
  This identifier is based on the Geo microformat: http://microformats.org/wiki/geo
 */
var S = require("js.spec");
var semantic_1 = require("../semantic");
var float_1 = require("./float");
var lists_1 = require("./lists");
function inDegreeRange(range) {
    return function degrees(value) { return value >= -range && value <= range; };
}
var latitudeSpec = S.spec.and("latitude", S.spec.number, inDegreeRange(90));
var longitudeSpec = S.spec.and("longitude", S.spec.number, inDegreeRange(180));
var geoLikeSpec = S.spec.map("GeoLike", {
    latitude: latitudeSpec,
    longitude: longitudeSpec
});
var geoArraySpec = S.spec.tuple("[lat, long]", latitudeSpec, longitudeSpec);
var geoInputSpec = S.spec.or("GeoInput spec", {
    "GeoLike": geoLikeSpec,
    "geo array": geoArraySpec
});
function isGeoLike(input) {
    return input.latitude !== undefined;
}
function forGeoIdentifier(input) {
    var latitude, longitude;
    if (isGeoLike(input)) {
        // Sadly object destructuring doesn't work with already-declared vars: {latitude, longitude} = input;
        latitude = input.latitude;
        longitude = input.longitude;
    }
    else {
        latitude = input[0], longitude = input[1];
    }
    return { latitude: latitude, longitude: longitude };
}
function toDebugString(value) {
    return "lat:" + value.latitude + "/long:" + value.longitude;
}
function encodeGeo(_a) {
    var latitude = _a.latitude, longitude = _a.longitude;
    return [float_1.floatCodec.encode(latitude), float_1.floatCodec.encode(longitude)];
}
function decodeGeo(_a) {
    var latitude = _a[0], longitude = _a[1];
    return { latitude: latitude, longitude: longitude };
}
exports.geoCodec = {
    type: "geo",
    typeCode: semantic_1.registerSemanticTypeCode(lists_1.LIST_TYPE_CODE | float_1.floatCodec.typeCode, 2),
    specForIdentifier: geoInputSpec,
    specForDecoding: geoArraySpec,
    forIdentifier: forGeoIdentifier,
    toDebugString: toDebugString,
    encode: encodeGeo,
    decode: decodeGeo,
};
//# sourceMappingURL=geo.js.map