"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Identifier-centric immutable wrapper around a Date instance. It provides the Date methods often used in system that
 * treat dates as identifiers. One can get a Date instance for other cases.
 */
var shared_1 = require("../shared");
/**
 * Creates an immutable date instance.
 * @param value can be either a timestamp number or a Date instance
 * @returns an ImmutableDate instance
 */
function createImmutableDate(value) {
    var time = shared_1.isNumber(value)
        ? value
        : value.getTime();
    var date = new Date(time);
    return Object.freeze({
        time: time,
        toString: function () { return date.toString(); },
        toUTCString: function () { return date.toUTCString(); },
        toISOString: function () { return date.toISOString(); },
        toJSON: function () { return date.toJSON(); },
        toDate: function () { return new Date(time); }
    });
}
exports.createImmutableDate = createImmutableDate;
//# sourceMappingURL=immutable-date.js.map