"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var S = require("js.spec");
var maps_1 = require("./maps");
exports.LIST_TYPE_CODE = 0x8;
exports.LIST_OF = 0x20;
function createListCodec(itemCodec) {
    var listTypeCode = calculateListTypeCode(itemCodec.typeCode);
    var listType = itemCodec.type + "-list";
    var forIdentifierListSpec = S.spec.and(listType + " forIdentifier spec", S.spec.array, // must be an array, not a Set
    S.spec.collection(listType + " item spec", itemCodec.specForIdentifier));
    var forDecodingListSpec = S.spec.and(listType + " forDecoding spec", S.spec.array, // must be an array, not a Set
    S.spec.collection(listType + " item spec", itemCodec.specForDecoding));
    function forIdentifier(list) {
        return list.map(itemCodec.forIdentifier);
    }
    function toDebugString(list) {
        var joined = list
            .map(function (value) { return itemCodec.toDebugString(value); })
            .join(", ");
        return "[" + joined + "]";
    }
    function encodeList(list) {
        return list.map(itemCodec.encode);
    }
    function decodeList(list) {
        return list.map(itemCodec.decode);
    }
    return {
        type: listType,
        typeCode: listTypeCode,
        specForIdentifier: forIdentifierListSpec,
        specForDecoding: forDecodingListSpec,
        forIdentifier: forIdentifier,
        toDebugString: toDebugString,
        encode: encodeList,
        decode: decodeList
    };
}
exports.createListCodec = createListCodec;
function calculateListTypeCode(itemTypeCode) {
    if ((itemTypeCode & maps_1.MAP_OF) === maps_1.MAP_OF) {
        throw new Error("Cannot create a List of Map of something. itemTypeCode: " + itemTypeCode);
    }
    if ((itemTypeCode & exports.LIST_OF) === exports.LIST_OF) {
        throw new Error("Cannot create a List of List of something. itemTypeCode: " + itemTypeCode);
    }
    var isStructural = itemTypeCode & exports.LIST_TYPE_CODE || itemTypeCode & maps_1.MAP_TYPE_CODE;
    return itemTypeCode | (isStructural
        ? exports.LIST_OF
        : exports.LIST_TYPE_CODE);
}
exports.calculateListTypeCode = calculateListTypeCode;
//# sourceMappingURL=lists.js.map