import * as Long from "long";
import { IdentifierCodec } from "../identifier-codec";
/**
 * 64 bit two's-complement integer, given its low and high 32 bit values as signed integers. Looks like a
 * Google Closure Long:
 * https://google.github.io/closure-library/api/goog.math.Long.html
 * https://github.com/google/closure-library/blob/master/closure/goog/math/long.js
 */
export interface LongLike {
    /**
     * The high 32 bits as a signed value.
     */
    readonly high: number;
    /**
     * The low 32 bits as a signed value.
     */
    readonly low: number;
    /**
     * Unsigned flag. Default is false.
     */
    unsigned?: boolean;
}
export declare type LongInput = number | LongLike;
/**
 * Encoded type of long. Can be either a number or Long.
 */
export declare type EncodedLong = number | Long;
export declare const longCodec: IdentifierCodec<LongInput, Long, EncodedLong>;
