"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
var S = require("js.spec");
var Long = require("long");
var integer_1 = require("./integer");
var shared_1 = require("../shared");
var shared_types_1 = require("./shared-types");
var longLikeSpec = S.spec.map("long value", (_a = {
        high: integer_1.integerSpec,
        low: integer_1.integerSpec
    },
    _a[S.symbol.optional] = {
        unsigned: S.spec.boolean
    },
    _a));
/*
  Convert a long-like input into a plain object. Must explicitly mask out 'unsigned' as js.spec will fail
  if key is defined, but value is undefined.
 */
function toPlainLongLike(_a) {
    var high = _a.high, low = _a.low, unsigned = _a.unsigned;
    return shared_1.exists(unsigned)
        ? { high: high, low: low, unsigned: unsigned }
        : { high: high, low: low };
}
var longInputSpec = S.spec.or("long input", {
    "number": Number.isInteger,
    "Long": Long.isLong,
    "LongLike": function (value) { return S.valid(longLikeSpec, toPlainLongLike(value)); }
});
var decodeSpec = S.spec.or("decoded long", {
    "number": integer_1.integerSpec,
    "Long": Long.isLong
});
/*
 Convert the input into a plain LongLike object, either from number or another LongLike object.
 */
function forIdentifier(input) {
    var long;
    if (Long.isLong(input)) {
        long = input;
    }
    else if (shared_1.isNumber(input)) {
        long = Long.fromNumber(input);
    }
    else {
        long = Long.fromBits(input.low, input.high, input.unsigned);
    }
    return long.toSigned();
}
function toDebugString(value) {
    return value.toString();
}
function decodeValue(encoded) {
    return shared_1.isNumber(encoded)
        ? Long.fromNumber(encoded)
        : encoded.toSigned();
}
exports.longCodec = {
    type: "long",
    typeCode: 0x4,
    specForIdentifier: longInputSpec,
    specForDecoding: decodeSpec,
    forIdentifier: forIdentifier,
    toDebugString: toDebugString,
    encode: shared_types_1.asIsCodec.encode,
    decode: decodeValue
};
//# sourceMappingURL=long.js.map