"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var S = require("js.spec");
var lists_1 = require("./lists");
exports.MAP_TYPE_CODE = 0x10;
exports.MAP_OF = 0x40;
function mapValues(map, mapFn, sortKeys) {
    var mapped = {};
    var keys = Object.keys(map);
    if (sortKeys) {
        // http://exploringjs.com/es6/ch_oop-besides-classes.html#_traversal-order-of-properties
        keys.sort();
    }
    for (var k = 0; k < keys.length; k++) {
        var key = keys[k];
        mapped[key] = mapFn(map[key]);
    }
    return mapped;
}
exports.mapValues = mapValues;
/**
 * Spec to apply an item spec to every value in a map.
 */
var MapValuesSpec = /** @class */ (function (_super) {
    __extends(MapValuesSpec, _super);
    function MapValuesSpec(itemSpec, prefix) {
        return _super.call(this, prefix + "(" + itemSpec + ")", { spec: itemSpec }) || this;
    }
    MapValuesSpec.prototype.itemSpec = function () {
        return this.options.spec;
    };
    MapValuesSpec.prototype.conform = function (value) {
        var keys = Object.keys(value);
        for (var k = 0; k < keys.length; k++) {
            var key = keys[k];
            if (!S.valid(this.itemSpec(), value[key])) {
                return S.symbol.invalid;
            }
        }
        return value;
    };
    MapValuesSpec.prototype.explain = function (path, via, value) {
        var problems = [];
        var keys = Object.keys(value);
        for (var k = 0; k < keys.length; k++) {
            var key = keys[k];
            var mapValue = value[key];
            if (!S.valid(this.itemSpec(), mapValue)) {
                problems = problems.concat(this.itemSpec().explain(path.concat([key]), via.concat([this.name]), mapValue));
            }
        }
        return problems;
    };
    return MapValuesSpec;
}(S.AbstractSpec));
exports.MapValuesSpec = MapValuesSpec;
function createMapCodec(itemCodec) {
    var mapType = itemCodec.type + "-map";
    var forIdentifierMapSpec = S.spec.and(mapType + " forIdentifier()", S.spec.object, new MapValuesSpec(itemCodec.specForIdentifier, "Map identifier values"));
    var forDecodingMapSpec = S.spec.and(mapType + " Map forDecoding()", S.spec.object, new MapValuesSpec(itemCodec.specForDecoding, "decoded Map values"));
    function forIdentifier(map) {
        return mapValues(map, itemCodec.forIdentifier, true);
    }
    function toDebugString(map) {
        var stringMap = mapValues(map, itemCodec.toDebugString);
        var keys = Object.keys(stringMap);
        var joined = keys
            .map(function (key) { return key + ": " + stringMap[key]; })
            .join(", ");
        return "{" + joined + "}";
    }
    function encodeMap(map) {
        return mapValues(map, itemCodec.encode);
    }
    function decodeMap(map) {
        return mapValues(map, itemCodec.decode);
    }
    return {
        type: mapType,
        typeCode: calculateMapTypeCode(itemCodec.typeCode),
        specForIdentifier: forIdentifierMapSpec,
        specForDecoding: forDecodingMapSpec,
        forIdentifier: forIdentifier,
        toDebugString: toDebugString,
        encode: encodeMap,
        decode: decodeMap
    };
}
exports.createMapCodec = createMapCodec;
function calculateMapTypeCode(itemTypeCode) {
    if ((itemTypeCode & exports.MAP_OF) === exports.MAP_OF) {
        throw new Error("Cannot create a Map of Map of something. itemTypeCode: " + itemTypeCode);
    }
    if ((itemTypeCode & lists_1.LIST_OF) === lists_1.LIST_OF) {
        throw new Error("Cannot create a Map of List of something. itemTypeCode: " + itemTypeCode);
    }
    var isStructural = itemTypeCode & lists_1.LIST_TYPE_CODE || itemTypeCode & exports.MAP_TYPE_CODE;
    return itemTypeCode | (isStructural
        ? exports.MAP_OF
        : exports.MAP_TYPE_CODE);
}
exports.calculateMapTypeCode = calculateMapTypeCode;
//# sourceMappingURL=maps.js.map